<?php

declare(strict_types=1);

namespace Dedi\SyliusSettingsPlugin\DependencyInjection\Compiler;

use Dedi\SyliusSettingsPlugin\DependencyInjection\Configuration;
use Dedi\SyliusSettingsPlugin\Security\Authorization\Checker\PluginSettingsAuthorizationChecker;
use Dedi\SyliusSettingsPlugin\Security\Authorization\CheckerRegistry;
use Symfony\Component\Config\Definition\Processor;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Reference;

class AuthorizationCheckerPass implements CompilerPassInterface
{
    public function process(ContainerBuilder $container)
    {
        $processor = new Processor();
        $configuration = $processor->processConfiguration(new Configuration(), $container->getExtensionConfig('dedi_sylius_settings'));

        $registryDefinition = $container->findDefinition(CheckerRegistry::class);

        foreach ($configuration['plugins'] as $pluginName => $configurationValues) {
            $authorizationCheckerId = 'dedi.sylius_settings.plugins.' . $pluginName . '.authorization_checker';
            $container
                ->register($authorizationCheckerId)
                ->setClass(PluginSettingsAuthorizationChecker::class)
                ->setArgument('$type', $configurationValues['permission'] ?? $pluginName)
                ->setArgument('$settingsProvider', new Reference($configurationValues['provider']))
            ;

            $registryDefinition->addMethodCall('addChecker', [new Reference($authorizationCheckerId)]);
        }
    }
}
